@@ -183,9 +183,9 @@ void IndexDlg::show()
     QDialog::show();
 }
 
-void IndexDlg::slotEntry( const char* name, int size, int mtime, int record )
+void IndexDlg::slotEntry( const QString & name, int size, int mtime, int record )
 {
-    if ( _fileName.length() > 0 ) {
+    if ( !_fileName.isEmpty() ) {
         _archive->addFile( _fileSize, _fileMTime, _fileStartRecord, record, _fileName );
     }
 
@@ -259,7 +259,7 @@ void IndexDlg::timerEvent( QTimerEvent* )
 
         delete _tarParser;
         _tarParser = new TarParser();
-        connect( _tarParser, SIGNAL( sigEntry( const char*, int, int, int ) ), this, SLOT( slotEntry( const char*, int, int, int ) ) );
+        connect( _tarParser, SIGNAL( sigEntry( const QString &, int, int, int ) ), this, SLOT( slotEntry( const QString &, int, int, int ) ) );
 
         int endBlock = 0;
         int leftover = 0;
@@ -271,10 +271,10 @@ void IndexDlg::timerEvent( QTimerEvent* )
                 _archiveCount++;
                 msg.setNum( _archiveCount );
                 _archives->setText( msg );
-                msg.sprintf( i18n( "Indexing archive %d." ), _archiveCount );
+                msg = i18n( "Indexing archive %1." ).arg( _archiveCount );
                 _log->append( msg );
 
-                msg.sprintf( i18n( "Archive %d" ), _archiveCount );
+                msg = i18n( "Archive %1" ).arg( _archiveCount );
                 _archive = new Archive( _tape, 0, msg );
                 _fileName = QString::null;
                 _fileSize = -1;
@@ -337,7 +337,7 @@ void IndexDlg::updateStats()
     QString str;
     int elapsed = time( NULL ) - _startTime;
 
-    str.sprintf( i18n( "%02d:%02d:%02d" ), elapsed / 3600, elapsed / 60 % 60, elapsed % 60 );
+    str = QString::fromUtf8( QCString().sprintf( i18n( "%02d:%02d:%02d" ).utf8(), elapsed / 3600, elapsed / 60 % 60, elapsed % 60 ) );
     _elapsedTime->setText( str );
 
     str = Util::kbytesToString( (int)_totalKBytes );
