@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////
-//      $Id: debInterface.h,v 1.8 2000/05/10 16:02:58 toivo Exp $ 
+//      $Id: debInterface.h,v 1.9 2000/07/14 15:58:15 toivo Exp $ 
 //
 // Author: Toivo Pedaste
 //
@@ -62,6 +62,8 @@ private:
   int doUninstall(int installFlags, QString packs);
   int doInstall(int installFlags, QString packs);
 
+  void distPackages(QList<packageInfo> *pki, cacheObj *cp);
+  void listUnIPackages(QList<packageInfo> *pki, LcacheObj *pCache);
 };
 
 #endif
