@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////         
-//      $Id: findf.cpp,v 1.10 2000/05/02 15:01:38 toivo Exp $ 
+//      $Id: findf.cpp,v 1.14 2000/08/28 20:05:54 waba Exp $ 
 //
 // Author: Toivo Pedaste
 //
@@ -16,8 +16,8 @@
 
 extern pkgInterface *kpinterface[];
 
-FindF::FindF(QWidget *parent, const char *name)
-    : QDialog(parent, name,FALSE){
+FindF::FindF(QWidget *parent)
+    : QDialog(parent, 0,FALSE){
 
     this->setFocusPolicy(QWidget::StrongFocus);
 
@@ -81,31 +81,33 @@ void FindF::doFind(const QString &str)
   tab->clear();
 
   for (i = 0; i < kpinterfaceN; i++) {
-    s = kpinterface[i]->FindFile(str);
+    if (kpinterface[i]) {
+      s = kpinterface[i]->FindFile(str);
 
-    if (s.length() > 0) {
-      cnt++;
-      int p = 0, pp = 0;
-      while ((p = s.find('\n',p)) >= 0) {
-	t = kpinterface[i]->head.data();
-	tmp = s.mid(pp,p - pp);
-	if (tmp.find("diversion by") >= 0) {
-	  new QListViewItem(tab, "", tmp);
+      if (s.length() > 0) {
+	cnt++;
+	int p = 0, pp = 0;
+	while ((p = s.find('\n',p)) >= 0) {
+	  t = kpinterface[i]->head;
+	  tmp = s.mid(pp,p - pp);
+	  if (tmp.find("diversion by") >= 0) {
+	    new QListViewItem(tab, "", tmp);
+	  }
+
+	  int t1 = tmp.find('\t');
+	  QString s1 = tmp.left(t1);
+	  QString s2 = tmp.right(tmp.length()-t1);
+
+	  new QListViewItem(tab, t, s1, s2);
+	  p++;
+	  pp = p;
 	}
-
-	int t1 = tmp.find('\t');
-	QString s1 = tmp.left(t1);
-	QString s2 = tmp.right(tmp.length()-t1);
-
-	new QListViewItem(tab, t, s1, s2);
-	p++;
-	pp = p;
       }
     }
-  }
 
-  if (!cnt) {
-    new QListViewItem(tab, "", i18n("--Nothing found--"));
+    if (!cnt) {
+      new QListViewItem(tab, "", i18n("--Nothing found--"));
+    }
   }
 }
 
@@ -126,7 +128,14 @@ void FindF::search(QListViewItem *item)
   if (p > 0) {
     s.truncate(p);
   }
-  kpackage->management->search(s ,item->text(0));
+ KPLVItem *k =  kpackage->management->search(s ,item->text(0));
+ if (k)
+   kpackage->management->changePack(k);
+}
+
+void FindF::dragEnterEvent(QDragEnterEvent* e)
+{
+  e->accept(QUriDrag::canDecode(e));
 }
 
 void FindF::dropEvent(QDropEvent *de) // something has been dropped
@@ -138,7 +147,7 @@ void FindF::dropEvent(QDropEvent *de) // something has been dropped
   const char *s = list.first();
   KURL *u = new KURL( s );
 
-  if (!u->isMalformed() && (strcmp( u->protocol(), "file" ) == 0 )) {
+  if (!u->isMalformed() && (u->protocol() == "file"  )) {
     url = u->path();
     if (url.right(1) == "/")
       url.truncate(url.length()-1);
