@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////
-//     $Id: rpmInstall.cpp,v 1.20 2000/05/29 16:17:21 toivo Exp $
+//     $Id: rpmInstall.cpp,v 1.26 2000/09/19 15:59:02 toivo Exp $
 #include "../config.h"
 
 #ifdef HAVE_RPM
@@ -24,16 +24,23 @@ extern "C"
 }
 
 static void printHash(const unsigned long amount, const unsigned long total);
+#if RPM_V1 < 4
 static void * showProgress(const Header h, const rpmCallbackType what, 
 			   const unsigned long amount, 
 			   const unsigned long total,
 			   const void * pkgKey, void * data);
+#else
+static void * showProgress(const void *hd, const rpmCallbackType what,
+			   const unsigned long amount,
+			   const unsigned long total,
+			   const void * pkgKey, void * data);
+#endif
 
 static int hashesPrinted = 0;
 
 static void printHash(const unsigned long amount, const unsigned long total)
 {
-    int hashesNeeded;
+    int hashesNeeded = -1;
 
     if (hashesPrinted != 50) {
 	hashesNeeded = (int)( 50 * (total ? (((float) amount) / total) : 1));
@@ -41,10 +48,18 @@ static void printHash(const unsigned long amount, const unsigned long total)
     kpackage->setPercent(hashesNeeded*2);   
 }
 
+#if RPM_V1 < 4
 static void * showProgress(const Header h, const rpmCallbackType what, 
 			   const unsigned long amount, 
 			   const unsigned long total,
 			   const void * pkgKey, void * data) {
+#else
+static void * showProgress(const void *hd, const rpmCallbackType what, 
+			   const unsigned long amount, 
+			   const unsigned long total,
+			   const void * pkgKey, void * data) {
+    Header h=(Header)hd;
+#endif
     char * s;
     int flags = (int) data;
     void * rc = NULL;
@@ -53,11 +68,19 @@ static void * showProgress(const Header h, const rpmCallbackType what,
 
     switch (what) {
       case RPMCALLBACK_INST_OPEN_FILE:
+#if RPM_V1 < 4
 	fd = fdOpen(filename, O_RDONLY, 0);
+#else /* RPM 4 */
+	fd = Fopen(filename, "r.ufdio");
+#endif
 	return fd;
 
       case RPMCALLBACK_INST_CLOSE_FILE:
+#if RPM_V1 < 4
 	fdClose(fd);
+#else /* RPM 4 */
+	Fclose(fd);
+#endif
 	break;
 
       case RPMCALLBACK_INST_START:
@@ -127,11 +150,11 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 	 defaultReloc++);
     if (defaultReloc && !defaultReloc->newPath) defaultReloc = NULL;
 
-    rpmMessage(RPMMESS_DEBUG, (char *) "%s\n", i18n("counting packages to install").ascii());
+    kdDebug() <<  i18n("counting packages to install") << endl;
 
     numPackages = argv.count();
 
-    rpmMessage(RPMMESS_DEBUG, (char *) "%s\n", i18n("found %1 packages").arg(numPackages).ascii() );
+    kdDebug() <<  i18n("found %1 packages").arg(numPackages) << endl;
 
     nb = (numPackages + 1) * sizeof(char *);
     packages = (const char **) alloca(nb);
@@ -148,38 +171,55 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 
     sourcePackages = (const char **)alloca(sizeof(*sourcePackages) * i);
 
-    rpmMessage(RPMMESS_DEBUG, (char *) "%s\n", i18n("retrieved %1 packages").arg(numTmpPackages).ascii());
+    kdDebug() << i18n("retrieved %1 packages").arg(numTmpPackages) << endl; 
 
     /* Build up the transaction set. As a special case, v1 source packages
        are installed right here, only because they don't have headers and
        would create all sorts of confusion later. */
 
     for (filename = packages; *filename; filename++) {
+#if RPM_V1 < 4
 	fd = fdOpen(*filename, O_RDONLY, 0);
 	if (fdFileno(fd) < 0) {
-	    rpmMessage(RPMMESS_ERROR, (char *) "%s\n", i18n("cannot open file %1").arg(*filename).ascii());
+	    KpMsgE(i18n("cannot open file %1").arg(*filename));
 	    numFailed++;
 	    packages[i] = NULL;
 	    continue;
 	}
+#else /* RPM 4 */
+	fd = Fopen(*filename, "r.ufdio");
+	if (Fileno(fd) < 0) {
+	    KpMsgE(i18n("cannot open file %1").arg(*filename));
+	    numFailed++;
+	    packages[i] = NULL;
+	    continue;
+	}
+#endif
 
 	rc = rpmReadPackageHeader(fd, &h, &isSource, &major, NULL);
 
 	switch (rc) {
 	case 1:
+#if RPM_V1 < 4
 	    fdClose(fd);
-	    rpmMessage(RPMMESS_ERROR, 
-		        (char *) "%s\n", i18n("%1 does not appear to be a RPM package").arg(*filename).ascii());
+#else /* RPM 4 */
+	    Fclose(fd);
+#endif
+	    KpMsgE(i18n("%1 does not appear to be a RPM package").arg(*filename));
 	    break;
 	default:
-	    rpmMessage(RPMMESS_ERROR, (char *) "%s\n", i18n("%1 cannot be installed").arg(*filename).ascii());
+	    KpMsgE(i18n("%1 cannot be installed").arg(*filename));
 	    numFailed++;
 	    packages[i] = NULL;
 	    break;
 	case 0:
 	    if (isSource) {
 		sourcePackages[numSourcePackages++] = *filename;
+#if RPM_V1 < 4
 		fdClose(fd);
+#else /* RPM 4 */
+		Fclose(fd);
+#endif
 	    } else {
 		if (!dbIsOpen) {
 		    if (rpmdbOpen(rootdir, &db, mode, 0644)) {
@@ -187,8 +227,7 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 		      //	dn = rpmGetPath( (rootdir ? rootdir : ""), 
 		      //			"%{_dbpath}", NULL);
 		      if (getuid() != 0)
-			rpmMessage(RPMMESS_ERROR, 
-				    (char *) "%s\n", i18n("KPACKAGE has to run as ROOT").ascii());
+			KpMsgE(i18n("KPACKAGE has to run as ROOT"),TRUE);
 			//			xfree(dn);
 			return(EXIT_FAILURE);
 		    }
@@ -208,8 +247,7 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 		    } else {
 			headerGetEntry(h, RPMTAG_NAME, NULL, (void **) &name,
 				       NULL);
-			rpmMessage(RPMMESS_ERROR, 
-			        (char *) "%s\n", i18n("package %1 is not relocateable").arg(name).ascii());
+			KpMsgE(i18n("package %1 is not relocateable").arg(name));
 
 			return numPackages;
 		    }
@@ -220,25 +258,27 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 			       relocations);
 		if (rc) {
 		    if (rc == 1)
-			rpmMessage(RPMMESS_ERROR, 
-			     (char *) "%s\n", i18n("error reading from file %1").arg(*filename).ascii());
+			KpMsgE(i18n("error reading from file %1").arg(*filename));
 		    else if (rc == 2)
-			rpmMessage(RPMMESS_ERROR, 
-			     (char *) "%s\n", i18n("file %1 requires a newer version of RPM").arg(*filename).ascii());
+			KpMsgE(i18n("file %1 requires a newer version of RPM").arg(*filename));
 		    return numPackages;
 		}
 
 		if (defaultReloc)
 		    defaultReloc->oldPath = NULL;
 
+#if RPM_V1 < 4
 		fdClose(fd);
+#else /* RPM 4 */
+		Fclose(fd);
+#endif
 		numBinaryPackages++;
 	    }
 	    break;
 	}
     }
 
-    rpmMessage(RPMMESS_DEBUG,  (char *) "%s\n", i18n("found %1 source and %2 binary packages").arg(numSourcePackages).arg(numBinaryPackages).ascii());
+    kdDebug() <<  i18n("found %1 source and %2 binary packages").arg(numSourcePackages).arg(numBinaryPackages) << endl;;
 
     if (numBinaryPackages && !(interfaceFlags & INSTALL_NODEPS)) {
 	if (rpmdepCheck(rpmdep, &conflicts, &numConflicts)) {
@@ -247,7 +287,6 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 	}
 
 	if (!stopInstall && conflicts) {
-	  //	    rpmMessage(RPMMESS_ERROR, (char *) "%s\n", i18n("failed dependencies:").ascii());
 	    KprintDepProblems(stderr, conflicts, numConflicts);
 	    rpmdepFreeConflicts(conflicts, numConflicts);
 	    numFailed = numPackages;
@@ -264,8 +303,8 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 
     if (numBinaryPackages && !stopInstall) {
 	rpmProblemSet probs = NULL;
-;
-	rpmMessage(RPMMESS_DEBUG,  (char *) "%s\n", i18n("installing binary packages").ascii());
+
+	kdDebug() << i18n("installing binary packages") << endl;
 	rc = rpmRunTransactions(rpmdep, showProgress, (void *) notifyFlags, 
 				    NULL, &probs, transFlags, probFilter);
 
@@ -276,7 +315,7 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 	    for (i = 0; i < probs->numProblems; i++) {
 		if (!probs->probs[i].ignoreProblem) {
 		    char *msg = (char *) rpmProblemString(probs->probs[i]);
-		    rpmMessage(RPMMESS_ERROR, (char *) "%s\n", "%s", msg);
+		    KpMsgE(msg);
 		    free(msg);
 		}
 	    }
@@ -290,17 +329,29 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 
     if (numSourcePackages && !stopInstall) {
 	for (i = 0; i < numSourcePackages; i++) {
+#if RPM_V1 < 4
 	    fd = fdOpen(sourcePackages[i], O_RDONLY, 0);
 	    if (fdFileno(fd) < 0) {
-		rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("cannot open file %1").arg(sourcePackages[i]).ascii());
+		KpMsgE(i18n("cannot open file %1").arg(sourcePackages[i]));
 		continue;
 	    }
+#else /* RPM 4 */
+	    fd = Fopen(sourcePackages[i], "r.ufdio");
+	    if (Fileno(fd) < 0) {
+		KpMsgE(i18n("cannot open file %1").arg(sourcePackages[i]));
+		continue;
+	    }
+#endif
 
 	    if (!(transFlags & RPMTRANS_FLAG_TEST))
 		numFailed += rpmInstallSourcePackage(rootdir, fd, NULL,
 				showProgress, (void *) notifyFlags, NULL);
 
-	    fdClose(fd);
+#if RPM_V1 < 4
+            fdClose(fd);
+#else /* RPM 4 */
+            Fclose(fd);
+#endif
 	}
     }
 
@@ -319,11 +370,17 @@ int doInstal(const char * rootdir, const QStringList &argv, int transFlags,
 int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
 		 int interfaceFlags) {
     rpmdb db;
+#if RPM_V1 < 4
     dbiIndexSet matches;
-    int i, j;
-    int mode;
+    int i;
     int rc;
     int count;
+#else
+    rpmdbMatchIterator mi;
+    Header h;
+#endif
+    int j;
+    int mode;
     int numFailed = 0;
     rpmTransactionSet rpmdep;
     struct rpmDependencyConflict * conflicts;
@@ -341,7 +398,7 @@ int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
 	const char *dn;
 	dn = rpmGetPath( (rootdir ? rootdir : ""), "%{_dbpath}", NULL);
         if (getuid() != 0)
-          rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("KPACKAGE has to run as ROOT").ascii(), dn);
+           KpMsgE(i18n("KPACKAGE has to run as ROOT"),TRUE);
 	xfree(dn);
 	return(EXIT_FAILURE);
     }
@@ -352,15 +409,16 @@ int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
         (it != argv.end());
         it++)
     {
-        const char *arg = (*it).ascii();
-        rc = rpmdbFindByLabel(db, arg, &matches);
+        QString arg = (*it);
+#if RPM_V1 < 4
+        rc = rpmdbFindByLabel(db, QFile::encodeName(arg), &matches);
 	switch (rc) {
 	case 1:
-            rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("package %1 is not installed").arg(arg).ascii());
+            KpMsgE(i18n("package %1 is not installed").arg(arg));
 	    numFailed++;
 	    break;
 	case 2:
-            rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("searching for package %1").arg(arg).ascii());
+            KpMsgE(i18n("searching for package %1").arg(arg));
 	    numFailed++;
 	    break;
 	default:
@@ -369,7 +427,7 @@ int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
 		if (dbiIndexRecordOffset(matches, i)) count++;
 
 	    if (count > 1 && !(interfaceFlags & UNINSTALL_ALLMATCHES)) {
-		rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("\"%1\" specifies multiple packages").arg(arg).ascii());
+		KpMsgE(i18n("\"%1\" specifies multiple packages").arg(arg));
 		numFailed++;
 	    }
 	    else { 
@@ -385,6 +443,22 @@ int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
 	    dbiFreeIndexRecord(matches);
 	    break;
 	}
+#else /* RPM v4 */
+	{
+	bool found=false;
+	mi=rpmdbInitIterator(db, RPMDBI_LABEL, QFile::encodeName(arg), 0);
+	while((h=rpmdbNextIterator(mi))!=NULL) {
+		found=true;
+		rpmtransRemovePackage(rpmdep, rpmdbGetIteratorOffset(mi));
+		numPackages++;
+	}
+	if(!found) {
+            KpMsgE(i18n("package %1 is not installed").arg(arg));
+	    numFailed++;
+	}
+	rpmdbFreeIterator(mi);
+	}
+#endif
     }
 
     if (!(interfaceFlags & UNINSTALL_NODEPS)) {
@@ -394,8 +468,6 @@ int doUninstal(const char * rootdir, const QStringList &argv, int transFlags,
 	}
 
 	if (!stopUninstall && conflicts) {
-	  //	    rpmMessage(RPMMESS_ERROR, (char *) "%s\n", i18n("removing these packages would break "
-	  //			      "dependencies:"));
 	    KprintDepProblems(stderr, conflicts, numConflicts);
 	    rpmdepFreeConflicts(conflicts, numConflicts);
 	    numFailed += numPackages;
@@ -419,24 +491,36 @@ int doSourceInstall(const char * rootdir, const char * arg, const char ** specFi
     FD_t fd;
     int rc;
 
+#if RPM_V1 < 4
     fd = fdOpen(arg, O_RDONLY, 0);
     if (fdFileno(fd) < 0) {
-	rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("cannot open %1").arg(arg).ascii());
+	KpMsgE(i18n("cannot open %1").arg(arg));
 	return 1;
     }
+#else /* RPM 4 */
+    fd = Fopen(arg, "r.ufdio");
+    if (Fileno(fd) < 0) {
+	KpMsgE(i18n("cannot open %1").arg(arg));
+	return 1;
+    }
+#endif
 
     if (rpmIsVerbose())
-      KpMsgE(i18n("Installing %1\n").arg(arg), FALSE);
+      kdDebug() << i18n("Installing %1\n").arg(arg) << endl;;
 
     rc = rpmInstallSourcePackage(rootdir, fd, specFile, NULL, NULL, 
 				 cookie);
     if (rc == 1) {
-	rpmMessage(RPMMESS_ERROR,  (char *) "%s\n", i18n("%1 cannot be installed").arg(arg).ascii());
+	KpMsgE(i18n("%1 cannot be installed").arg(arg),TRUE);
 	if (specFile) FREE(*specFile);
 	if (cookie) FREE(*cookie);
     }
 
+#if RPM_V1 < 4
     fdClose(fd);
+#else /* RPM 4 */
+    Fclose(fd);
+#endif
 
     return rc;
 }
@@ -487,13 +571,13 @@ void KprintDepProblems(FILE * f, struct rpmDependencyConflict * conflicts,
 	}
 
 	if (conflicts[i].sense == (rpmDependencyConflict::RPMDEP_SENSE_REQUIRES)) { 
-	    fprintf(f, i18n(" is needed by %s-%s-%s\n"), conflicts[i].byName, 
+	    fprintf(f, i18n(" is needed by %s-%s-%s\n").local8Bit(), conflicts[i].byName, 
 		    conflicts[i].byVersion, conflicts[i].byRelease);
             st += s = i18n(" is needed by %1-%2-%3\n")
                              .arg(conflicts[i].byName)
                     .arg(conflicts[i].byVersion).arg(conflicts[i].byRelease);
 	} else {
-	    fprintf(f, i18n(" conflicts with %s-%s-%s\n"), conflicts[i].byName, 
+	    fprintf(f, i18n(" conflicts with %s-%s-%s\n").local8Bit(), conflicts[i].byName, 
 		    conflicts[i].byVersion, conflicts[i].byRelease);
             st += s = i18n(" conflicts with %1-%2-%3\n")
                             .arg(conflicts[i].byName)
@@ -504,7 +588,7 @@ void KprintDepProblems(FILE * f, struct rpmDependencyConflict * conflicts,
        st.truncate(1000);
        st+="...";
      }
-    rpmMessage(RPMMESS_ERROR, (char *) "%s\n",st.ascii());	
+    KpMsgE(st);	
 }
 
 extern "C"
@@ -520,7 +604,7 @@ extern "C"
 
   QString *rpmEBuf = 0;
 
-  void rpmErr() {
+  void rpmErr(void) {
     if (rpmEBuf) {
       *rpmEBuf += rpmErrorString();
       *rpmEBuf += "\n";
