@@ -3,13 +3,11 @@
 #ifndef PREFERENCES_DIALOG_H
 #define PREFERENCES_DIALOG_H
 
-#pragma interface
-
-#include <kdialogbase.h>
-#include <qcolor.h>
 #include <qstring.h>
 
-class KDualColorButton;
+#include <kdualcolorbtn.h>
+#include <kdialogbase.h>
+
 class KLineEdit;
 class KSVConfig;
 class QPushButton;
@@ -22,15 +20,18 @@ public:
   static KSVPreferences* self ();
   virtual ~KSVPreferences ();
 
-  QColor newNormal () const;
-  QColor newSelected () const;
-  QColor changedNormal () const;
-  QColor changedSelected () const;
+  inline QColor newNormal () const { return mNewColors->foreground(); }
+  inline QColor newSelected () const { return mNewColors->background(); }
+  inline QColor changedNormal () const { return mChangedColors->foreground(); }
+  inline QColor changedSelected () const { return mChangedColors->background(); }
 
   QString runlevelPath () const;
   QString scriptPath () const;
 
-  inline bool aColorChanged () const { return mColorsChanged; }
+signals:
+  void updateColors ();
+  void updateServicesPath ();
+  void updateRunlevelsPath ();
 
 protected slots:
   virtual void slotApply ();
@@ -40,12 +41,15 @@ protected slots:
 private slots:
   void scriptDir();
   void runlevelDir();
-  void colorChanged();
   void reReadConfig();
+  void showAllMessages ();
 
 private:
+  friend class KSVTopLevel;
+
   KSVPreferences (QWidget* parent);
   void setConfig ();
+  bool checkPaths();
 
   KSVConfig* mConfig;
 
@@ -57,8 +61,6 @@ private:
 
   QPushButton* mBrowseScripts;
   QPushButton* mBrowseRunlevels;
-
-  bool mColorsChanged;
 };
 
 
