@@ -1,14 +1,11 @@
-#pragma implementation
-
 /*
-
     Requires the Qt widget libraries, available at no cost at
     http://www.troll.no
 
     Copyright (C) 1997-2000 Peter Putzer
                             putzer@kde.org
 
-	$Id: TopWidget.cpp,v 1.52 2000/06/07 21:35:33 putzer Exp $
+	$Id: TopWidget.cpp,v 1.91 2000/10/08 11:24:04 putzer Exp $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of version 2 of the GNU General Public License
@@ -22,7 +19,6 @@
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
 */
 
 /****************************************************************
@@ -32,17 +28,21 @@
 **
 ****************************************************************/
 
+#include <ctype.h>
+
 #include <qpopmenu.h>
 #include <qkeycode.h>
 #include <qmsgbox.h>
-#include <qfontmetrics.h>
-#include <qpainter.h>
+// #include <qfontmetrics.h>
+// #include <qpainter.h>
 #include <qmultilinedit.h>
 #include <qdatetime.h>
-#include <qprinter.h>
+// #include <qprinter.h>
+// #include <qpaintdevicemetrics.h>
 #include <qbuttongroup.h>
 #include <qclipboard.h>
-#include <qprintdialog.h>
+// #include <qprintdialog.h>
+#include <qtooltip.h>
 #include <qcheckbox.h>
 #include <qhbox.h>
 #include <qregexp.h>
@@ -51,11 +51,12 @@
 #include <qtextview.h>
 #include <qfileinfo.h>
 #include <qtextstream.h>
-#include <qprogressdialog.h>
+#include <qwhatsthis.h>
 
 #include <ktoolbar.h>
 #include <kkeydialog.h>
 #include <kmenubar.h>
+#include <kcompletion.h>
 #include <kaction.h>
 #include <kstdaction.h>
 #include <kdebug.h>
@@ -68,6 +69,7 @@
 #include <kedittoolbar.h>
 #include <kiconloader.h>
 #include <kfiledialog.h>
+#include <kaboutdata.h>
 
 #include "ServiceDlg.h"
 #include "PreferencesDialog.h"
@@ -77,27 +79,33 @@
 #include "About.h"
 #include "RunlevelAuthIcon.h"
 #include "ksvdraglist.h"
-#include "Constants.h"
+#include "Data.h"
+#include "ksvdrag.h"
 #include "ConfigWizard.h"
 #include "OldView.h"
+#include "IOCore.h"
 #include "TopWidget.h"
 
-
-struct Status
+namespace Status
 {
   enum {
-    Changed, Progress, Checklist, Writable
+    Changed, Checklist, Writable
   };
-};
+}; // namespace Status
 
 KSVTopLevel::KSVTopLevel()
-  : KTMainWindow(),
+  : KMainWindow(0, 0L, WStyle_ContextHelp|WDestructiveClose),
     mConfig(KSVConfig::self()),
 	mView (0L),
-    mPreferences(0L),
+    mPreferences (0L),
+
 	mEditUndo (0L), mEditCut (0L), mEditCopy (0L), mEditPaste (0L),
-	mFileNew (0L), mFilePrintLog (0L), mFileSave (0L), mFileQuit (0L),
-	mToolsStartService (0L), mToolsStopService (0L), mToolsRestartService (0L), mToolsEditService (0L),
+	mFileRevert (0L), mFileLoad (0L), mFilePrint (0L), mFilePrintLog (0L),
+    mFileSave (0L), mFileSaveAs (0L), mFileSaveLog(0L), mFileQuit (0L),
+
+	mToolsStartService (0L), mToolsStopService (0L),
+    mToolsRestartService (0L), mToolsEditService (0L),
+
 	mOptionsTools (0L), mOptionsStatus (0L), mOptionsToggleLog (0L),
     mUndoList (new ActionList (this, "UndoList")),
 	mRedoList (new ActionList (this, "RedoList")),
@@ -113,9 +121,7 @@ KSVTopLevel::KSVTopLevel()
 	mEditDlg (new ServiceDlg (i18n("Edit Service"),
 							  i18n("&Choose which service to edit:"),
 							  this)),
-	mAdvanced (0),
-	mProgress (new QProgressDialog ("", QString::null, // is set later
-									10, this, 0L, true))
+    mVisible (new QCheckBox*[ksv::runlevelNumber])
 {
   kapp->setMainWidget (this);
 
@@ -126,24 +132,19 @@ KSVTopLevel::KSVTopLevel()
   initTools(); // order dependency
   initStatusBar(); // order dependency
 
-  setView(mView, false);
+  setCentralWidget(mView);
 
   // ensure that the statusbar gets updated correctly
   connect (mView, SIGNAL(sigRun(const QString&)), this, SLOT(slotUpdateRunning(const QString&)));
   connect (mView, SIGNAL(sigStop()), statusBar(), SLOT(clear()));
   connect (mView, SIGNAL(cannotGenerateNumber()), this, SLOT(catchCannotGenerateNumber()));
   connect (mView, SIGNAL(sigChanged()), this, SLOT(slotChanged()));
-  connect (mView, SIGNAL(undoAction(KSVAction*)), mUndoList, SLOT(push(KSVAction*)));
+  connect (mView, SIGNAL(undoAction(KSVAction*)), this, SLOT(pushUndoAction(KSVAction*)));
   connect (mView, SIGNAL(logChanged()), this, SLOT(enableLogActions()));
 
   // cut & copy
-  connect (mView, SIGNAL(selected(KSVItem*)),
-		   this, SLOT(dispatchCutCopy(KSVItem*)));
-
-  connect (mView, SIGNAL(selectedScripts(KSVItem*)),
-		   this, SLOT(dispatchCopyScripts(KSVItem*)));
-
-  dispatchCutCopy(0L); // disable cut & copy on startup
+  connect (mView, SIGNAL (newOrigin()),
+		   this, SLOT (dispatchEdit()));
 
   // undo
   connect (mUndoList, SIGNAL(empty()), this, SLOT(disableUndo()));
@@ -158,43 +159,30 @@ KSVTopLevel::KSVTopLevel()
 
   // paste
   connect (kapp->clipboard(), SIGNAL (dataChanged()),
-		   this, SLOT (dispatchPaste()));
-  dispatchPaste(); // disable paste on startup
+		   this, SLOT (dispatchEdit()));
 
   // init mView according to saved preferences
   slotReadConfig();
   initView();
-
   setMinimumSize(600,400);
 
   // restore size and position
   move(mConfig->position()); // doesnt seem to work while unmapped
   resize(mConfig->size());
 
-  // restore toolbar position
-  //  tools_->setBarPos(conf->getToolPosition());
-  //  menu_->setMenuBarPos(conf->getMenuPosition());
+//   // set enabled or disabled, depending on AuthIcon
+//   mView->multiplexEnabled (mAuth->status());
+  mAuth->setCheckEnabled(true);
 
-  // set enabled or disabled, depending on AuthIcon
-  mView->multiplexEnabled (mAuth->status());
-
-  // progress dialog
-  mProgress->setAutoReset (true);
-  mProgress->setAutoClose (true);
-  mProgress->setCaption (kapp->makeStdCaption (i18n("Clearing Configuration")));
-
-  // make progress bar update itself
-  connect (mView, SIGNAL(initProgress(int, const QString&)),
-		   this, SLOT(initProgress(int, const QString&)));
-  connect (mView, SIGNAL (advance(int)), this, SLOT (fwdAdvance(int)));
-  connect (this, SIGNAL (progress (int)), mProgress, SLOT (setProgress (int)));
-  connect (mView, SIGNAL(endProgress()), mProgress, SLOT (reset()));
+  dispatchEdit(); // disable cut & copy on startup
 }
 
 KSVTopLevel::~KSVTopLevel()
 {
   mUndoList->clear();
   mRedoList->clear();
+
+  delete[] mVisible;
 }
 
 void KSVTopLevel::initTools()
@@ -222,31 +210,32 @@ void KSVTopLevel::initTools()
 void KSVTopLevel::initActions ()
 {
   // setup File menu
-  mFileNew = KStdAction::openNew(this, SLOT(slotClearChanges()), actionCollection());
-  mFileNew->setText (i18n("&New Configuration..."));
+  mFileRevert = KStdAction::revert (this, SLOT (slotClearChanges()), actionCollection());
+  mFileRevert->setText (i18n("Re&vert Configuration..."));
+
+  mFileLoad = KStdAction::open (this, SLOT (load()), actionCollection());
+  mFileLoad->setText (i18n ("&Open..."));
 
   mFileSave = KStdAction::save(this, SLOT(slotAcceptChanges()), actionCollection());
   mFileSave->setText (i18n("&Save Configuration..."));
 
+  mFileSaveAs = KStdAction::saveAs (this, SLOT (saveAs ()), actionCollection());
+
   mFileSaveLog = KStdAction::save (this, SLOT(slotSaveLog()), actionCollection(), "ksysv_save_log");
-  //  mFileSaveLog = KStdAction::saveAs (this, SLOT(slotSaveLog()), actionCollection());
   mFileSaveLog->setText (i18n("Save &Log..."));
   mFileSaveLog->setAccel (Key_L+CTRL);
-  //  mFileSaveLog = new KAction (i18n("Save &Log..."), QIconSet (BarIcon("filesave")), 0, this,
-  //							  SLOT(slotSaveLog()), actionCollection(), "ksysv_save_log");
+  mFileSaveLog->setEnabled (false);
+
+  // disabled due to complexity
+  //  mFilePrint = KStdAction::print (this, SLOT (print()), actionCollection());
+
+//   mFilePrintLog = KStdAction::print(this, SLOT(printLog()), actionCollection(), "ksysv_print_log");
+//   mFilePrintLog->setText( i18n("&Print Log..."));
+//   mFilePrintLog->setEnabled (false);
 
-  //KStdAction::saveAs(this, SLOT(slotSaveLog()), actionCollection(), "my_save")->setText("Save &Log...");
-//   //  KStdAction::close(this, SLOT(file_close()), actionCollection());
-  mFilePrintLog = KStdAction::print(this, SLOT(printLog()), actionCollection(), "ksysv_print_log");
-  mFilePrintLog->setText( i18n("&Print Log..."));
-//   //  KStdAction::mail(this, SLOT(mail()), actionCollection());
-//   //  (void)new KAction(i18n("Open &URL..."), 0, this, SLOT(file_open_url()),
-// //   //				actionCollection(), "file_open_url");
-// //   (void)new KAction(i18n("Save &To URL..."), 0, this, SLOT(file_save_url()),
-// // 					actionCollection(), "save_to_url");
   mFileQuit = KStdAction::quit(this, SLOT(close()), actionCollection());
   
-  // setup edit menu
+  // setup Edit menu
   mEditUndo = KStdAction::undo(this, SLOT(editUndo()), actionCollection());
   mEditRedo = KStdAction::redo(this, SLOT(editRedo()), actionCollection());
   mEditCut = KStdAction::cut(this, SLOT(editCut()), actionCollection());
@@ -254,15 +243,6 @@ void KSVTopLevel::initActions ()
   mEditPaste = KStdAction::paste(this, SLOT(editPaste()), actionCollection());
   mPasteAppend = KStdAction::paste (this, SLOT (pasteAppend()),
 									actionCollection(), "ksysv_paste_append");
-
-  //  KStdAction::selectAll(this, SLOT(select_all()), actionCollection());
-
-//   new KAction (i18n("&Test"), 0, this, SLOT(print()), actionCollection(), "my_print");
-//   // setup Tools menu
-// //   KStdAction::spelling(this, SLOT(spellcheck()), actionCollection());
-  
-// //   // setup Go menu
-// //   KStdAction::gotoLine(this, SLOT(gotoLine()), actionCollection());
   
   // setup Settings menu
   mOptionsTools = KStdAction::showToolbar(this, SLOT(toggleTools()), actionCollection());
@@ -275,12 +255,7 @@ void KSVTopLevel::initActions ()
 										 this, SLOT (toggleLog()),
 										 actionCollection (), "ksysv_toggle_log");
 
-  // edit menu
-//   ToolsMenu_->insertItem(i18n("&Start Service..."), mStartDlg, SLOT(show()));
-//   ToolsMenu_->insertItem(i18n("S&top Service..."), mStopDlg, SLOT(show()));
-//   ToolsMenu_->insertItem(i18n("&Restart Service..."), remStartDlg, SLOT(show()));
-//   ToolsMenu_->insertSeparator();
-//   ToolsMenu_->insertItem(i18n("&Edit Service..."), mEditDlg, SLOT(show()));
+  // setup Tools menu
   mToolsStartService = new KToggleAction (i18n("&Start Service..."), "ksysv_start", 0,
 										 mStartDlg, SLOT (show()),
 										 actionCollection(), "ksysv_start_service");
@@ -311,7 +286,7 @@ bool KSVTopLevel::queryExit() {
 
   if (mChanged) {
     res = KMessageBox::warningYesNo(kapp->mainWidget(),
-				    i18n("There are unsaved changes.\nAre you sure you want to quit?"),
+				    i18n("There are unsaved changes. Are you sure you want to quit?"),
 				    i18n("Quit"),
 				    i18n("&Quit"),
 				    i18n("&Cancel"));
@@ -327,9 +302,9 @@ void KSVTopLevel::slotClearChanges()
   if (mChanged && 
       KMessageBox::Yes ==
       KMessageBox::questionYesNo(kapp->mainWidget(),
-				 i18n("Do you really want to clear all unsaved changes?"),
-				 i18n("New Configuration"),
-				 i18n("C&lear"),
+				 i18n("Do you really want to revert all unsaved changes?"),
+				 i18n("Revert Configuration"),
+				 i18n("&Revert"),
 				 i18n("&Cancel")))
     {
       mUndoList->undoAll();
@@ -341,8 +316,8 @@ void KSVTopLevel::slotAcceptChanges() {
   if (KMessageBox::Yes ==
       KMessageBox::warningYesNo(kapp->mainWidget(),
 				i18n("You're about to save the changes made to your init "
-				     "configuration.\nPlease realize that wrong settings could "
-				     "result in your system being unbootable.\n\n"
+				     "configuration.\nPlease realize that wrong settings can "
+				     "make your system hang on startup.\n\n"
 				     "Do you wish to continue?"),
 				i18n("Save Configuration"),
 				i18n("&Save"),
@@ -355,10 +330,15 @@ void KSVTopLevel::slotAcceptChanges() {
 
 void KSVTopLevel::initView()
 {
-  dispatchCutCopy(0L); // disabling cut&copy
+  const bool authEnabled = mAuth->isCheckEnabled();
+  mAuth->setCheckEnabled(false);
+
   mUndoList->clear();
   mRedoList->clear();
-  mView->reInit();
+
+  mView->initScripts();
+  mView->initRunlevels();
+
   setChanged(false);
 
   // disable ToolsMenu_ entries when they can't do anything
@@ -395,80 +375,102 @@ void KSVTopLevel::initView()
   mStopDlg->resetChooser (mView->scripts, false);
   mRestartDlg->resetChooser (mView->scripts, false);
   mEditDlg->resetChooser (mView->scripts, true);
+
+  mAuth->setCheckEnabled(authEnabled);
 }
 
 void KSVTopLevel::initStatusBar()
 {
   KStatusBar* status = statusBar();
 
-  QHBox* visBox = new QHBox (status);
-  QButtonGroup* group = new QButtonGroup (this);
+  QHBox* visBox = new QHBox (status, "visBox");
+  QButtonGroup* group = new QButtonGroup (this, "visButtonGroup");
   group->hide();
   connect (group, SIGNAL (clicked (int)), this, SLOT (toggleRunlevel (int)));
 
-  new QLabel (i18n(" Show Runlevels "), visBox);
-  for (int i = 0; i < RUNLEVEL_NR; ++i)
+  QWhatsThis::add (visBox, i18n ("<p>Click on the checkboxes to <strong>show</strong> or "\
+                                 "<strong>hide</strong> runlevel panes.</p> " \
+                                 "<p>The list of currently visible runlevels is saved "\
+                                 "when you use the <strong>Save Options command</strong>.</p>"));
+  QToolTip::add (visBox, i18n ("Show/Hide Runlevels"));
+
+  new QLabel (i18n(" Show Runlevels "), visBox, "visLabel");
+  for (int i = 0; i < ksv::runlevelNumber; ++i)
 	{
 	  QString label; label.setNum (i);
 
-	  mVisible[i] = new QCheckBox (label, visBox, label);
+	  mVisible[i] = new QCheckBox (label, visBox, label.latin1());
 	  mVisible[i]->setChecked (mConfig->showRunlevel (i));
 	  
 	  group->insert (mVisible[i]);
 	}
 
-  //  mProgress->setMinimumWidth(130);
-
-  QHBox* authIconBox = new QHBox (status);
-  QWidget* strut = new QWidget (authIconBox);
+  QHBox* authIconBox = new QHBox (status, "AuthIconBox");
+  QWidget* strut = new QWidget (authIconBox, "Strut");
   strut->setFixedWidth (KDialog::spacingHint());
   mAuth = new RunlevelAuthIcon (mConfig->scriptPath(), mConfig->runlevelPath(), authIconBox);
   connect (mAuth, SIGNAL (authChanged(bool)), mView, SLOT(multiplexEnabled(bool)));
+  connect (mAuth, SIGNAL (authChanged(bool)), this, SLOT(writingEnabled(bool)));
 
-  status->addWidget (authIconBox, Status::Writable);
-  status->insertItem (i18n(" Changed"), Status::Changed, 100);
-  //  status->addWidget(mProgress, Status::Progress);
-  status->addWidget (visBox, Status::Checklist);
+  QWhatsThis::add (authIconBox, i18n ("<p>When the lock is closed <img src=\"user|ksysv_locked\"/>, "\
+                                      "you haven't got the right " \
+                                      "<strong>permissions</strong> to edit the init configuration.</p>" \
+                                      "<p>Either restart %1 as root (or another more privileged user), " \
+                                      "or ask your sysadmin to install %1 <em>suid</em> or " \
+                                      "<em>sgid</em>.</p><p>The latter way is <strong>not</strong> "\
+                                      "recommended though, due to security issues.</p>")
+                   .arg (kapp->aboutData()->programName()).arg(kapp->aboutData()->programName()));
+
+  authIconBox->setMinimumSize (authIconBox->minimumSizeHint());
+  visBox->setMinimumSize (visBox->minimumSizeHint());
+
+  status->addWidget (authIconBox, 0, false);
+  status->insertItem ("", Status::Changed, 100);
+  status->addWidget (visBox, 0, true);
   
-  status->setItemAlignment (Status::Changed, AlignLeft | AlignVCenter);
-//   status->setAlignment( Status::Progress, AlignRight );
-//   status->setAlignment( Status::Rest, AlignRight );
-
-  status->changeItem("", Status::Changed);
-  //  statusBar()->changeItem("", Status::Rest);
+  status->setItemAlignment (Status::Changed, AlignLeft|AlignVCenter);
 }
 
 void KSVTopLevel::slotShowConfig()
 {
-  // FIXME
+  if (!mPreferences)
+    {
+      mPreferences = KSVPreferences::self();
+      
+      connect (mPreferences, SIGNAL (updateColors ()),
+               this, SLOT (updateColors ()));
 
-  KSVPreferences* test = KSVPreferences::self();
-  test->exec();
-//   if (!mPreferences)
-//     (mPreferences = new KSVPrefDlg(this, "Preferences"))->hide();
+      connect (mPreferences, SIGNAL (updateServicesPath ()),
+               this, SLOT (updateServicesPath ()));
 
-//   mPreferences->reReadConfig();
-  
-//   if (mPreferences->exec() == QDialog::Accepted)
-// 	{
-	  
-// 	  mConfig->setScriptPath( mPreferences->scriptPath() );
-// 	  mConfig->setRunlevelPath( mPreferences->runlevelPath() );
-	  
-// 	  if (mPreferences->isColorChanged())
-// 		{
-// 		  mConfig->setNewColor(mPreferences->newColor());
-// 		  mConfig->setChangedColor(mPreferences->changedColor());
-		  
-// 		  mView->setNewColor(mConfig->newColor());
-// 		  mView->setChangedColor(mConfig->changedColor());
-// 		  QPixmapCache::clear(); // unfortunately, this is necessary
-// 		  mView->repaintRunlevels(false);
-// 		}
-	  
-// 	}
-//   else
-//     mPreferences->reReadConfig();
+      connect (mPreferences, SIGNAL (updateRunlevelsPath ()),
+               this, SLOT (updateRunlevelsPath ()));
+    }
+
+  mPreferences->setInitialSize (QSize (400,300), true);
+  mPreferences->exec();
+}
+
+void KSVTopLevel::updateColors ()
+{
+  ksv::serviceCompletion ()->clear ();
+
+  mView->setColors (mConfig->newNormalColor(),
+                    mConfig->newSelectedColor(),
+                    mConfig->changedNormalColor(),
+                    mConfig->changedSelectedColor());
+}
+
+void KSVTopLevel::updateServicesPath ()
+{
+  mView->updateServicesAfterChange (mConfig->scriptPath());
+  mAuth->setServicesPath (mConfig->scriptPath());
+}
+
+void KSVTopLevel::updateRunlevelsPath ()
+{
+  mView->updateRunlevelsAfterChange ();
+  mAuth->setRunlevelPath (mConfig->runlevelPath());
 }
 
 void KSVTopLevel::slotChanged() {
@@ -487,18 +489,11 @@ void KSVTopLevel::toggleLog() {
 }
 
 void KSVTopLevel::toggleStatus() {
-  setStatus (!statusBar()->isVisible());
+  setStatus ( mOptionsStatus->isChecked() );
 }
 
 void KSVTopLevel::toggleTools() {
-  setTools (!toolBar()->isVisible());
-}
-
-void KSVTopLevel::fwdAdvance (int i)
-{
-  mAdvanced += i;
-
-  emit progress (mAdvanced);
+  setTools ( mOptionsTools->isChecked() );
 }
 
 void KSVTopLevel::saveOptions()
@@ -566,15 +561,13 @@ void KSVTopLevel::editPaste()
 void KSVTopLevel::setChanged (bool val)
 {
   mChanged = val;
-  // for session management
   setCaption(val);
 
-  mFileNew->setEnabled (val);
+  mFileRevert->setEnabled (val);
   mFileSave->setEnabled (val);
 
   // update statusbar
-  QString barTxt = val ? i18n(" Changed") : QString::null;
-  statusBar()->changeItem(barTxt, Status::Changed);
+  statusBar()->changeItem(val ? i18n(" Changed") : QString::null, Status::Changed);
 
   // clear messages
   statusBar()->clear();
@@ -585,7 +578,7 @@ void KSVTopLevel::properties()
   KSVDragList* list = mView->getOrigin();
 
   if (list)
-    mView->infoOnData(dynamic_cast<KSVItem*>(list->currentItem()));
+    mView->infoOnData(list->currentItem());
 }
 
 void KSVTopLevel::scriptProperties()
@@ -640,7 +633,8 @@ void KSVTopLevel::setRedo (bool val)
   mEditRedo->setEnabled (val);
 }
 
-void KSVTopLevel::setLog( bool val ) {
+void KSVTopLevel::setLog (bool val)
+{
   mConfig->setShowLog(val);
   
   mOptionsToggleLog->setChecked (val);
@@ -648,7 +642,8 @@ void KSVTopLevel::setLog( bool val ) {
   mView->setDisplayScriptOutput(val);
 }
 
-void KSVTopLevel::setStatus( bool val ) {
+void KSVTopLevel::setStatus (bool val)
+{
   if (val)
 	statusBar()->show();
   else
@@ -658,53 +653,163 @@ void KSVTopLevel::setStatus( bool val ) {
   mOptionsStatus->setChecked (val);
 }
 
-void KSVTopLevel::setTools( bool val ) {
+void KSVTopLevel::setTools (bool val)
+{
   if (val)
-	toolBar()->show();
+	toolBar("mainToolBar")->show();
   else
-	toolBar()->hide();
+	toolBar("mainToolBar")->hide();
 
 
   mConfig->setShowTools (val);
   mOptionsTools->setChecked (val);
 }
 
-void KSVTopLevel::initProgress (int max, const QString& label)
+void KSVTopLevel::writingEnabled (bool on)
 {
-  mAdvanced = 0;
-  mProgress->setTotalSteps (max);
-  mProgress->setLabelText (label);
-  mProgress->setFixedSize (mProgress->sizeHint());
-  mProgress->show();
-
-  emit progress (mAdvanced);
+  mFileLoad->setEnabled(on);
 }
 
 void KSVTopLevel::print()
 {
-  kdDebug() << "Printing not implemented." << endl;
+// #define checkPage if (metrics.height() - y < fm.lineSpacing()) prt.newPage();
+
+//   static QPrinter prt;
+//   prt.setDocName(kapp->aboutData()->programName() + " Configuration");
+//   prt.setCreator(kapp->aboutData()->programName());
+
+//   static QPrintDialog* dlg = new QPrintDialog (&prt, this, "KSysV Print Dialog");
+//   dlg->setCaption(kapp->makeStdCaption (i18n("Print")));
+  
+//   if (dlg->exec() == QDialog::Accepted)
+//     {
+//       int y = 10;
+//       QPainter p;
+//       p.begin( &prt );
+
+//       QPaintDeviceMetrics metrics (&prt);
+  
+//       p.setFont (QFont("courier", 20, QFont::Bold));
+//       QFontMetrics fm = p.fontMetrics();
+
+//       p.drawText (10, y, i18n("%1 Configuration of %2")
+//                   .arg (kapp->aboutData()->programName())
+//                   .arg (ksv::hostname()));
+//       y += fm.lineSpacing();
+
+//       p.drawText (10, y, QDateTime::currentDateTime().toString());
+//       y += fm.lineSpacing() * 2; // an extra empty line
+
+//       for (int i = 0; i < ksv::runlevelNumber; ++i)
+//         {
+//           p.setFont (QFont("courier", 16, QFont::Bold));
+//           QFontMetrics fm = p.fontMetrics();
+          
+//           p.drawText (10, y, i18n ("Runlevel %1").arg(i));
+//           y += fm.lineSpacing();
+
+//           checkPage
+          
+//           p.drawText (10, y, i18n ("Started Services"));
+//           y += fm.lineSpacing() * 2; // an extra empty line
+
+//        checkPage
+            
+//           p.setFont (QFont("courier", 10));
+//           fm = p.fontMetrics();
+
+//           for (QListViewItemIterator it (mView->startRL[i]);
+//                it.current();
+//                ++it)
+//             {
+//               KSVItem* item = static_cast<KSVItem*> (it.current());
+
+//               y += fm.ascent();
+//               p.drawText (10, y, item->toString());
+//               y += fm.descent();
+
+//        checkPage
+
+//             }
+
+//           p.setFont (QFont("courier", 16, QFont::Bold));
+//           fm = p.fontMetrics();
+//           y += fm.lineSpacing(); // an extra empty line
+//           p.drawText (10, y, i18n ("Stopped Services"));
+//           y += fm.lineSpacing() * 2; // an extra empty line
+   
+//        checkPage
+       
+//           p.setFont (QFont("courier", 10));
+//           fm = p.fontMetrics();
+
+//           for (QListViewItemIterator it (mView->stopRL[i]);
+//                it.current();
+//                ++it)
+//             {
+//               KSVItem* item = static_cast<KSVItem*> (it.current());
+//        checkPage
+
+//               y += fm.ascent();
+//               p.drawText (10, y, item->toString());
+//               y += fm.descent();
+//             }
+//        checkPage
+
+//           p.setFont (QFont("courier", 16, QFont::Bold));
+//           fm = p.fontMetrics();
+//           y += fm.lineSpacing() * 3; // two extra empty line
+//         }
+//       //      QStringList lines = QStringList::split ('\n', mView->log(), true);
+//       //       for(QStringList::Iterator it = lines.begin();
+//       //           it != lines.end();
+//       //           ++it)
+//       //         {
+//       //           y += fm.ascent();
+          
+//       //           QString line = *it;
+//       //           if (line.isNull())
+//       //             line = " ";
+
+//       //           line.replace( QRegExp("\t"), "        " );
+
+//       //           strncpy(buf,line.data(),160);
+          
+//       //           for (int j = 0 ; j <150; j++)
+//       //             {
+//       //               if (!isprint(buf[j]))
+//       //                 buf[j] = ' ';
+//       //             }
+          
+//       //           buf[line.length()] = '\0';
+//       //           p.drawText( 10, y, buf );
+//       //           y += fm.descent();
+//       //         }
+      
+//       p.end();        
+//     }
 }
 
 void KSVTopLevel::printLog()
 {  
 //   static QPrinter prt;
-//   prt.setDocName(kapp->caption() + " Log File");
-//   prt.setCreator(kapp->caption());
+//   prt.setDocName(kapp->aboutData()->programName() + " Log File");
+//   prt.setCreator(kapp->aboutData()->programName());
 
-//   static QPrintDialog* dlg = new QPrintDialog (&prt, this, "KSysV Print Dialog");
+//   static QPrintDialog* dlg = new QPrintDialog (&prt, this, "KSysV Print Log Dialog");
 //   dlg->setCaption(kapp->makeStdCaption (i18n("Print Log")));
   
 //   char buf[200];
-//   if ( dlg->exec() == QDialog::Accepted )
+//   if (dlg->exec() == QDialog::Accepted)
 //     {
-//       int y =10;
+//       int y = 10;
 //       QPainter p;
 //       p.begin( &prt );
   
 //       p.setFont (QFont("courier", 10, QFont::Bold));
 //       QFontMetrics fm = p.fontMetrics();
 
-//       p.drawText (10, y, "KDE System V Init Editor Log");
+//       p.drawText (10, y, i18n("%1 Log").arg(kapp->aboutData()->programName()));
 //       y += fm.lineSpacing();
 
 //       p.drawText (10, y, QDateTime::currentDateTime().toString());
@@ -712,31 +817,39 @@ void KSVTopLevel::printLog()
 
 //       p.setFont (QFont("courier", 10));
 //       fm = p.fontMetrics();
-      
-//       const int numlines = mView->logLines();
-//       for(int i = 0; i< numlines; i++){
-// 	y += fm.ascent();
-	
-// 	QString line = mView->logLine(i);
-// 	line.replace( QRegExp("\t"), "        " );
-// 	strncpy(buf,line.data(),160);
-	
-// 	for (int j = 0 ; j <150; j++)
-// 	  {
-// 	    if (!isprint(buf[j]))
-// 	      buf[j] = ' ';
-// 	  }
-	
-// 	buf[line.length()] = '\0';
-// 	p.drawText( 10, y, buf );
-// 	y += fm.descent();
-//       }
+
+//       QStringList lines = QStringList::split ('\n', mView->log(), true);
+//       for(QStringList::Iterator it = lines.begin();
+//           it != lines.end();
+//           ++it)
+//         {
+//           y += fm.ascent();
+          
+//           QString line = *it;
+//           if (line.isNull())
+//             line = " ";
+
+//           line.replace( QRegExp("\t"), "        " );
+
+//           strncpy(buf,line.data(),160);
+          
+//           for (int j = 0 ; j <150; j++)
+//             {
+//               if (!isprint(buf[j]))
+//                 buf[j] = ' ';
+//             }
+          
+//           buf[line.length()] = '\0';
+//           p.drawText( 10, y, buf );
+//           y += fm.descent();
+//         }
       
 //       p.end();        
 //     }
 }
 
-void KSVTopLevel::catchCannotGenerateNumber() {
+void KSVTopLevel::catchCannotGenerateNumber()
+{
   statusBar()->message(i18n("Could not generate sorting number. Please change manually."), 5000);
 }
 
@@ -749,45 +862,49 @@ void KSVTopLevel::closeEvent (QCloseEvent* e)
       return;
     }
 
-  KTMainWindow::closeEvent (e);
+  KMainWindow::closeEvent (e);
 }
 
-void KSVTopLevel::dispatchCutCopy (KSVItem* d)
+void KSVTopLevel::dispatchEdit ()
 {
-  if (!d)
+  KSVDragList* list = mView->getOrigin();
+
+  if (!list)
     {
-      setCopy(false);
-      setCut(false);
-    }
-  else
-    {
-      setCopy(true);
-      setCut(true);
+      setCopy (false);
+      setCut (false);
+      setPaste (false);
+
+      return;
     }
 
-  dispatchPaste();
-}
+  KSVItem* current = list->currentItem();
 
-void KSVTopLevel::dispatchCopyScripts (KSVItem* d)
-{
-  if (!d)
-    setCopy(false);
+  if (current)
+	{
+	  setCopy (true);
+	  setCut (true);
+	}
   else
-    setCopy(true);
-
-  setCut(false);
+	{
+	  setCopy (false);
+	  setCut (false);
+	}
   
-  dispatchPaste();
-}
-
-void KSVTopLevel::dispatchPaste ()
-{
-  QMimeSource* mime = kapp->clipboard()->data();
-
-  if (mime && mime->provides ("application/x-ksysv") && !mView->scripts->isOrigin())
-	setPaste (true);
+  if (mView->scripts == list)
+	{
+	  setCut (false);
+	  setPaste (false);
+	}
   else
-	setPaste (false);
+	{
+	  QMimeSource* mime = kapp->clipboard()->data();
+
+	  if (mime && mime->provides ("application/x-ksysv"))
+		setPaste (true);
+	  else
+		setPaste (false);
+	}
 }
 
 void KSVTopLevel::enableUndo()
@@ -813,14 +930,14 @@ void KSVTopLevel::disableRedo ()
 
 void KSVTopLevel::slotSaveLog()
 {
-  static QString filter = "*.log|" + QString(kapp->caption() + i18n(" log files") + " (*.log)");
+  static QString filter = "*.ksysv_log|" + QString(kapp->caption() + i18n(" log files") + " (*.ksysv_log)");
 
   QString filename = KFileDialog::getSaveFileName(0L, filter, this);
 
   if (filename.isEmpty())
     return;
-  else if (filename.right(4) != ".log")
-    filename += ".log";
+  else if (filename.right(4) != ".ksysv_log")
+    filename += ".ksysv_log";
 
   QFile file(filename);
 
@@ -839,7 +956,7 @@ void KSVTopLevel::slotSaveLog()
 
 void KSVTopLevel::show()
 {
-  KTMainWindow::show();
+  KMainWindow::show();
 
 #if 0 // we need this in the future...
 
@@ -864,7 +981,7 @@ void KSVTopLevel::show()
 void KSVTopLevel::enableLogActions ()
 {
   mFileSaveLog->setEnabled (true);
-  mFilePrintLog->setEnabled (true);
+  //  mFilePrintLog->setEnabled (true); // disabled due to complexity
 }
 
 void KSVTopLevel::checkScriptMenu (QPopupMenu* menu, int row)
@@ -897,7 +1014,7 @@ void KSVTopLevel::checkScriptMenu (QPopupMenu* menu, int row)
 
 void KSVTopLevel::setCaption (bool changed)
 {
-  setPlainCaption (kapp->makeStdCaption(KSVCore::hostname(), true, changed));
+  setPlainCaption (kapp->makeStdCaption(ksv::hostname(), true, changed));
 }
 
 void KSVTopLevel::configureKeys ()
@@ -911,7 +1028,7 @@ void KSVTopLevel::configureKeys ()
 
 void KSVTopLevel::configureToolbars ()
 {
-  KEditToolbar* dlg = new KEditToolbar (actionCollection(), xmlFile(), this);
+  KEditToolbar* dlg = new KEditToolbar (actionCollection(), xmlFile(), true, this);
 
   if (dlg->exec())
 	{
@@ -957,3 +1074,69 @@ void KSVTopLevel::toggleRunlevel (int index)
 
   mConfig->setShowRunlevel (index, state);
 }
+
+void KSVTopLevel::saveAs ()
+{
+  static QString filter = "*.ksysv|" + i18n("Saved Init Configurations") + " (*.ksysv)";
+  QString filename = KFileDialog::getSaveFileName(0L, filter, this);
+
+  if (filename.isEmpty())
+    return;
+  else if (filename.right(6) != ".ksysv")
+    filename += ".ksysv";
+
+  QFile file(filename);
+
+  file.open (IO_WriteOnly | IO_Raw);
+  QDataStream s (&file);
+
+  if (ksv::IO::saveConfiguration (s, mView->startRL, mView->stopRL))
+    {
+      statusBar()->message(i18n("Configuration package saved successfully."), 5000);
+    }
+  else
+    {
+      kdDebug(3000) << "ERROR saving file" << endl;
+    }
+
+  file.close();
+}
+
+void KSVTopLevel::load ()
+{  
+  static QString filter = "*.ksysv|" + i18n("Saved Configurations") + " (*.ksysv)";
+  QString filename = KFileDialog::getOpenFileName(0L, filter, this);
+
+  if (filename.isEmpty())
+    return;
+
+  QFile file(filename);
+
+  file.open (IO_ReadOnly | IO_Raw);
+  QDataStream s (&file);
+  QValueList<KSVData>* startLists = new QValueList<KSVData>[ksv::runlevelNumber];
+  QValueList<KSVData>* stopLists = new QValueList<KSVData>[ksv::runlevelNumber];
+  
+  if (ksv::IO::loadSavedConfiguration (s, startLists, stopLists))
+    {
+      statusBar()->message(i18n("Configuration package loaded successfully."), 5000);
+      mView->mergeLoadedPackage (startLists, stopLists);
+    }
+  else
+    {
+      kdDebug (3000) << "ERROR loading file" << endl;
+    }
+
+  file.close();
+
+  delete[] startLists;
+  delete[] stopLists;
+}
+
+void KSVTopLevel::pushUndoAction (KSVAction* action)
+{
+  mRedoList->clear();
+  mUndoList->push (action);
+}
+
+#include "TopWidget.moc"
