@@ -13,6 +13,8 @@ int quotactl(int cmd, const char * special, int id, caddr_t addr) {
 #  endif
 #endif
 
+#include <qfile.h>
+
 #include "misc.h"
 #include "kglobal_.h"
 #include "mnt.h"
@@ -145,23 +147,23 @@ static int doQuotaCtl(int ACmd, uint AUID, const MntEnt *m, struct dqblk *dq) {
   qctl.uid = AUID;
   qctl.addr = (caddr_t)dq;
     
-  fd = open(m->getquotafilename(), (ACmd == _KU_GETQUOTA) ? O_RDONLY : O_WRONLY);
+  fd = open(QFile::encodeName(m->getquotafilename()), (ACmd == _KU_GETQUOTA) ? O_RDONLY : O_WRONLY);
   int res = ioctl(fd, Q_QUOTACTL, &qctl);
   close(fd);
   return res;
 #else
 #  if defined(_KU_EXT2_QUOTA) || defined(HAVE_IRIX)
-         return quotactl(ACmd, m->getfsname(), AUID, (caddr_t) dq);
+         return quotactl(ACmd, QFile::encodeName(m->getfsname()), AUID, (caddr_t) dq);
 #  else
 #    ifdef __osf__
-         return quotactl((const char*) m->getdir(), ACmd, AUID, (caddr_t) dq);
+         return quotactl((char*)QFile::encodeName(m->getdir()).data(), ACmd, AUID, (caddr_t) dq);
 #    else
 #        ifdef BSD
-             return quotactl(m->getdir(), ACmd, AUID, (caddr_t) dq);
+             return quotactl(QFile::encodeName(m->getdir()), ACmd, AUID, (caddr_t) dq);
 #        endif
 #    endif
 #    ifdef _KU_HPUX_QUOTA
-         return quotactl(ACmd, m->getquotafilename(), AUID, dq);
+         return quotactl(ACmd, QFile::encodeName(m->getquotafilename()), AUID, dq);
 #    endif
 #  endif
 #endif
